SUBROUTINE EMTESF(FILEIN,FILEOUT,NFLDSKIP,LMASK,LXFLD,LYFLD,XMIN,XMAX,YMIN,YMAX, &
                  &IOUTFLD,FRMT,HEADING,BOUNDFILE,CHANFILE,COLFILE,SURVFILE,WIREFILE,WAVEFFILE)
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!
!
!   DUMPS SELECTED FIELDS IN BDB FILE TO AN ESF FILE
!
! KMF 8/12
! - added support files to output - 11/13 - kmf
! - fixed bug with azimuth and dip output from HOLE_ANGLES field - 11/14 - kmf
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
USE WINTERACTER
USE INPPARAMS

IMPLICIT NONE
CHARACTER(LEN=255), INTENT(IN) :: FILEIN  !INTPUT FILE NAME
CHARACTER(LEN=255), INTENT(IN) :: FILEOUT !OUTPUT FILE NAME
INTEGER, INTENT(IN) :: NFLDSKIP           !FIELD TO LOOK FOR NULLS IN - IF 0 NO CHECKING DONE
LOGICAL, INTENT(IN) :: LMASK              !FLAG TO MASK OUTPUT
INTEGER, INTENT(IN) :: LXFLD              !FIELD FOR X IF MASKED OUTPUT
INTEGER, INTENT(IN) :: LYFLD              !FIELD FOR Y IF MASKED OUTPUT
REAL,  INTENT(IN) :: XMIN                 !X & Y LIMITS IF MASKED
REAL, INTENT(IN) :: XMAX
REAL, INTENT(IN) :: YMIN
REAL, INTENT(IN) :: YMAX
INTEGER, DIMENSION(MAXFLDS), INTENT(INOUT) :: IOUTFLD !ARRAY OF FLAGS TO OUTPUT OR NOT
CHARACTER(LEN=8),INTENT(INOUT) :: FRMT    !FORTRAN FORMAT SPECIFIER - 1 FORMAT FITS ALL
CHARACTER(LEN=80),INTENT(IN) :: HEADING   !DATA DESCRIPTOR FOR HEADER LINE 1
CHARACTER(LEN=255), INTENT(IN) :: BOUNDFILE !BOUNDARY FILE FOR ESF OUTPUT
CHARACTER(LEN=255), INTENT(IN) :: CHANFILE  !CHANNEL FILE FOR ESF OUTPUT
CHARACTER(LEN=255), INTENT(IN) :: COLFILE   !COLLAR FILE FOR ESF OUTPUT
CHARACTER(LEN=255), INTENT(IN) :: SURVFILE  !SURVEY FILE FOR ESF OUTPUT
CHARACTER(LEN=255), INTENT(IN) :: WIREFILE  !WIRE/LOOP FILE FOR ESF OUTPUT
CHARACTER(LEN=255), INTENT(IN) :: WAVEFFILE !WAVEFORM FILE FOR ESF OUTPUT

CHARACTER(LEN=500) :: OUTMESS
CHARACTER(LEN=2000) :: OUTSTR
CHARACTER(LEN=20), PARAMETER :: OUTSUB='EMTESF'
CHARACTER(LEN=30) :: TEMPSTR
CHARACTER(LEN=255) :: SUPPFILE              !NAME FOR SUPPORT FILE
CHARACTER(LEN=1), DIMENSION(3), PARAMETER :: DHCOMP = (/'U','V','A'/)
CHARACTER(LEN=1), DIMENSION(3), PARAMETER :: SURCOMP = (/'X','Y','Z'/)
INTEGER :: LUHIN
INTEGER :: MODE,I,IOUT,J
INTEGER :: IACTION
INTEGER :: ISTART,IEND
INTEGER :: ISUPFILE                         !COUNTER FOR NUMBER OF SUPPORT FILES NEEDED
INTEGER :: LUHSUPF
!INTEGER :: IASTAT
REAL :: AZMTH,AINCL                       !AZIMUTH AND DIP FOR DOWNPOINTS READINGS

CALL IUPPERCASE(FRMT)
ISTART=INDEX(FRMT,'G')
IF (ISTART == 0) THEN
  ISTART=INDEX(FRMT,'F')
END IF
IF (ISTART == 0) THEN
  OUTMESS='THERE IS NO G OR F IN YOUR FORMAT SPECIFIER. ONLY G OR F FORMATS ARE ALLOWED'
  CALL WERRORBOX(OUTMESS,OUTSUB)
END IF
IEND=INDEX(FRMT,'.')
IF (IEND == 0) THEN
  OUTMESS='THERE IS NO DECIMAL POINT IN YOUR FORMAT SPECIFIER.'
  CALL WERRORBOX(OUTMESS,OUTSUB)
END IF
!
!     WRITE ACTION TO PRT FILE
WRITE(LUHPRT,905)TRIM(FILEIN),TRIM(FILEOUT)
905 FORMAT(/,'CONVERTING DATABASE TO AN ESF FILE',/,'INPUT: ',A,/,'OUTPUT: ',A)
IF (LMASK) THEN
  IF (LXFLD < 1  .OR. LYFLD < 1) THEN
    OUTMESS='YOU HAVE SPECIFIED MASKING OF THE OUTPUT BUT NOT SET THE FIELD TO MASK WITH!'
    CALL WERRORBOX(OUTMESS,OUTSUB)
  END IF
  !
  !     CHECK CRITERIA FIELDS EXIST
  IF (LXFLD > NFLD .OR. LYFLD > NFLD) THEN
    OUTMESS='YOU HAVE SPECIFIED A MASKING FIELD GREATER THAT THE NUMBER OF FIELDS IN THIS DATABASE'
    CALL WERRORBOX(OUTMESS,OUTSUB)
  END IF
  WRITE(LUHPRT,'("BASED ON THE FOLLOWING CONDITION:")')
  WRITE(LUHPRT,910) LXFLD,XMIN,XMAX
  WRITE(LUHPRT,910) LYFLD,YMIN,YMAX
  910 FORMAT(' FIELD ',I4,' MINIMUM =',G15.7,' MAXIMUM =',G15.7)
END IF
! COUNT THE NUMBER OF SUPPORT FILES NEEDED AND IF > 1 ALLOW FOR A COMPOSITE FILE
ISUPFILE=0
IF (LEN_TRIM(BOUNDFILE) > 0) ISUPFILE=ISUPFILE+1
IF (LEN_TRIM(CHANFILE) > 0) ISUPFILE=ISUPFILE+1
IF (LEN_TRIM(COLFILE) > 0) ISUPFILE=ISUPFILE+1
IF (LEN_TRIM(SURVFILE) > 0) ISUPFILE=ISUPFILE+1
IF (LEN_TRIM(WIREFILE) > 0) ISUPFILE=ISUPFILE+1
IF (LEN_TRIM(WAVEFFILE) > 0) ISUPFILE=ISUPFILE+1
IF (ISUPFILE > 0) THEN
  IF (ISUPFILE > 1) THEN
    SUPPFILE=FILEOUT(1:INDEX(FILEOUT,'.',.TRUE.))//'CON'
  ELSEIF(LEN_TRIM(BOUNDFILE) > 0) THEN
    SUPPFILE=FILEOUT(1:INDEX(FILEOUT,'.',.TRUE.))//'BND'
  ELSEIF(LEN_TRIM(CHANFILE) > 0) THEN
    SUPPFILE=CHANFILE
  ELSEIF(LEN_TRIM(COLFILE) > 0) THEN
    SUPPFILE=COLFILE
  ELSEIF(LEN_TRIM(SURVFILE) > 0) THEN
    SUPPFILE=SURVFILE
  ELSEIF(LEN_TRIM(WIREFILE) > 0) THEN
    SUPPFILE=FILEOUT(1:INDEX(FILEOUT,'.',.TRUE.))//'TXW'
  ELSEIF(LEN_TRIM(WAVEFFILE) > 0) THEN
    SUPPFILE=WAVEFFILE
  END IF
  CALL GETHANDL(LUHSUPF)
  OPEN(UNIT=LUHSUPF,FILE=SUPPFILE,ACCESS='SEQUENTIAL',FORM='FORMATTED',STATUS='UNKNOWN')
  SUPPFILE=SUPPFILE(INDEX(SUPPFILE,'\',.TRUE.)+1:LEN_TRIM(SUPPFILE))
END IF
!
!     OPEN INPUT DATABASE AND GET HEADER
MODE=1
CALL BDBOPN(LUHIN,MODE,FILEIN)
CALL BDBHDR(LUHIN,MODE,NFLD,LOCX,LOCY,NPTS,LSORT,LABEL,ZMIN,ZMAX,LTYPE)
CALL BDBPRH(LUHIN,LUHPRT,FILEIN)
CALL BDBXHDR(LUHIN,MODE,HARR,HSTR)
!
!CHECK FOR EMPTY FILE
!
IF(NPTS .LE. 0) THEN
  OUTMESS='ERROR - BINARY DATABASE IS EMPTY '
  CALL WERRORBOX(OUTMESS,OUTSUB)
END IF
!
!  GET NUMBER OF WINDOWS
CALL LABTIME()
!
!check that ramp time is in mSec
IF (HARR(12) > 3) THEN
  WRITE(OUTMESS,'("In Jan 2012 I changed the ramp time to mSec to be consistent with all other times. &
   &The header currently has a value of ",g12.5," which looks like uSec. If it is answer yes to this dialogue. &
   &Divide ramp time by 1000?")')HARR(12)
  CALL WYESNOBOX(OUTMESS,OUTSUB,IACTION)
  IF (IACTION==1) THEN
    HARR(12)=HARR(12)/1000.0
    MODE=2
    CALL BDBXHDR(LUHIN,MODE,HARR,HSTR)
  END IF    
END IF
!
! REPLACE ANY SPACES IN THE LABELS
DO I=1,NFLD
  CALL LETSUB(' ','_',LABEL(I)(1:LEN_TRIM(LABEL(I))))
END DO  
!
!     OPEN OUTPUT FILE
CALL GETHANDL(LUHOUT)
OPEN(UNIT=LUHOUT,FILE=FILEOUT,ACCESS='SEQUENTIAL',FORM='FORMATTED',STATUS='UNKNOWN')
CALL ESFEMHEAD()
OUTSTR=''
!
!   GET STARTED
IOUT=0
CALL BDBRRB(-1,1,ZDATA,NFLD,NPTS)
DO I=1,NPTS
  !
  !   INPUT RECORD
  CALL BDBRRB(LUHIN,I,ZDATA,NFLD,NPTS)
  !
  !PRINT PROGRESS MESSAGE
  IF(MOD(I,1000) .EQ. 0) THEN
    CALL BDSTATUS(I,NPTS,ZNULL)
  END IF
  ! HAVE TO EMBED THE FOLLOWING TWO IF CHECKS TO AVOID REQUEST FOR ZDATA(0)
  ! TEST IF CHECKING FOR NULLS AND IF SO IS IT NULL?
  IF (NFLDSKIP >0 ) THEN
    IF (LNULL(ZDATA(NFLDSKIP))) CYCLE
  END IF
  !
  !TEST IF TRIMMING OUTPUT TO DEFINED AREA AND IF SO ARE WE IN THAT AREA?
  IF (LMASK) THEN
    IF((ZDATA(LXFLD).GE.XMAX).OR.(ZDATA(LXFLD).LE.XMIN).OR.(ZDATA(LYFLD).GE.YMAX).OR. &
          (ZDATA(LYFLD).LE.YMIN)) CYCLE
  END IF
  OUTSTR=''
  DO J=1,NFLD
    IF (IOUTFLD(J) > 0) THEN
      IF (J == IANG .AND. TRIM(LABEL(IANG)) == 'HOLE_ANGLES') THEN
        AZMTH = FLOAT(INT(ZDATA(IANG)/1000.0))
        AINCL = ZDATA(IANG) - 1000.0*AZMTH
        WRITE(TEMPSTR,'(F6.1)')AZMTH
        OUTSTR=TRIM(OUTSTR)//TEMPSTR
        WRITE(TEMPSTR,'(F6.1)')-AINCL
        OUTSTR=TRIM(OUTSTR)//TEMPSTR
      ELSEIF (J == ICOMP) THEN                             !COMPONENT
        IF (TRIM(HSTR(10)) =='DH') THEN
          TEMPSTR=DHCOMP(INT(ZDATA(J)))
        ELSE
          TEMPSTR=SURCOMP(INT(ZDATA(J)))
        END IF
        OUTSTR=TRIM(OUTSTR)//' '//TEMPSTR
      ELSE  
        WRITE(TEMPSTR,FRMT)ZDATA(J)
        IF (IOUTFLD(J) == 3) THEN                   !DATE YYMMDD
          IF (ZDATA(J) > 600000) THEN               !ASSUME NO EM DATA BEFORE 1960
            TEMPSTR=" 19"//ADJUSTL(TEMPSTR)
          ELSE
            TEMPSTR=" 20"//ADJUSTL(TEMPSTR)
          END IF
        END IF  
        OUTSTR=TRIM(OUTSTR)//TEMPSTR
      END IF  
    END IF  
  END DO
  IOUT=IOUT+1
  WRITE(LUHOUT,'(A)')TRIM(OUTSTR)
END DO
IF (IOUT.EQ.0) THEN
  OUTMESS='NO POINTS FOUND WHICH FIT CRITERIA'
  CALL WINFOBOX(OUTMESS,OUTSUB)
END IF
WRITE(LUHPRT,'(/,"NUMBER OF POINTS OUTPUT: ",I10)')IOUT
!
!
CLOSE (LUHIN)
CLOSE (LUHOUT)
! create support files
IF (ISUPFILE >0) CALL CREATESUPPORT()

RETURN
CONTAINS
  SUBROUTINE ESFEMHEAD()
  !----------------------------------------------------------------------------------
  !
  ! PRINTS OUT EXTENDED HEADER AS A SERIES OF COMMENTS
  !
  !----------------------------------------------------------------------------------
  INTEGER :: I
  CHARACTER(LEN=200) :: TEMPSTR
  CHARACTER(LEN=14), DIMENSION(13), PARAMETER :: HEADSTR = (/'COMPANY       ', &
   &'AREA          ','PROJECT       ','CONTRACTOR    ','              ','CONFIG        ', &
   &'TRANSMITTER   ','RECEIVER      ','SENSOR        ','LINE_DIRECTION','UNITS.LENGTH  ', &
   &'              ','UNITS.EMIP    '/)
  CHARACTER(LEN=20), DIMENSION(14), PARAMETER :: HEADNUM = (/'LOOP                ', &
    &'TRANSMITTER_SERIAL_#','RECEIVER_SERIAL_#   ','SENSOR_SERIAL_#     ',&
    &'DUMMY               ','DUMMY               ','DUMMY               ',&
    &'DUMMY               ','GRID.DECLINATION    ','MAG.DECLINATION     ',&
    &'APPROX_DATE         ','TURNOFF             ','RXAREA              ',&
    &'TXFREQ              '/)
  CHARACTER(LEN=9), DIMENSION(9), PARAMETER :: ESFLABEL = (/'DUMMY    ', &
    &'BARALT   ','DATE     ','GAIN     ','GPSALT   ','NSTACK   ','RADALT   ', &
    &'TURNOFF  ','INITDELAY'/)
  
  !
  !   OUTPUT HEADER INFO
  WRITE(LUHOUT,'("VER:0001 ",A," ESF FILE EXPORTED FROM BINARY FILE ",A)')TRIM(HEADING),TRIM(FILEIN)
  !CREATE RECORD TYPE 2
  WRITE(TEMPSTR,FMT=FRMT)ZNULL
  IF (NCHAN > 0) THEN
    OUTSTR='DATATYPE:TEM NULL:'
  ELSE
    OUTSTR='DATATYPE:FEM NULL:'
  END IF    
  OUTSTR=TRIM(OUTSTR)//TRIM(ADJUSTL(TEMPSTR))
  DO I=1,13
    IF (LEN_TRIM(HSTR(I)) > 0 .AND. LEN_TRIM(HEADSTR(I)) > 0) THEN
      IF (INDEX(HSTR(I)(1:LEN_TRIM(HSTR(I))),' ') > 0 ) CALL LETSUB(' ','_',HSTR(I)(1:LEN_TRIM(HSTR(I))))
      IF (I == 6) THEN
        CALL IUPPERCASE(HSTR(I))
        IF (INDEX(HSTR(I),'IN_LOOP') > 0 ) THEN
          HSTR(I)='INLOOP'
        ELSEIF (INDEX(HSTR(I),'COINCIDENT') > 0 ) THEN 
          HSTR(I)='COINCIDENT'
        ELSEIF (INDEX(HSTR(I),'DOWN_HOLE') > 0 ) THEN 
          HSTR(I)='DOWNPOINTS'
        ELSEIF (INDEX(HSTR(I),'FIXED_LOOP') > 0 ) THEN 
          HSTR(I)='FIXEDLOOP'
        ELSEIF (INDEX(HSTR(I),'FIGURE_8') > 0 ) THEN 
          HSTR(I)='INOUT'
        END IF
      ELSEIF (I == 11)THEN
        CALL IUPPERCASE(HSTR(I))
        IF (INDEX(HSTR(I),'METRES') > 0 ) THEN
          HSTR(I)='M'
        ELSEIF (INDEX(HSTR(I),'FEET') > 0 ) THEN 
          HSTR(I)='FT'
        END IF
      END IF  
      WRITE(TEMPSTR,'(A,":",A)')TRIM(HEADSTR(I)),TRIM(HSTR(I))
      IF (LEN_TRIM(OUTSTR) > 0 ) THEN
        OUTSTR=TRIM(OUTSTR)//' '//TEMPSTR
      ELSE
        OUTSTR=TRIM(OUTSTR)//TEMPSTR
      END IF      
      IF (LEN_TRIM(OUTSTR) > 80) THEN
        WRITE(LUHOUT,'(A)')TRIM(OUTSTR)
        OUTSTR=''
      END IF
    END IF
  END DO
  DO I=1,14
    IF (.NOT. LNULL(HARR(I)) .AND. LEN_TRIM(HEADNUM(I)) > 0) THEN
      SELECT CASE (I)
        CASE(1)
          IF (TRIM(HSTR(6)) == 'SLINGRAM') THEN
            TEMPSTR='RXDX:'
            IF (LEN_TRIM(OUTSTR) > 0 ) THEN
              OUTSTR=TRIM(OUTSTR)//' '//TEMPSTR
            ELSE
              OUTSTR=TRIM(OUTSTR)//TEMPSTR
            END IF      
            WRITE(TEMPSTR,'(G15.7)')HARR(I)
            OUTSTR=TRIM(OUTSTR)//ADJUSTL(TEMPSTR)
          ELSE  
            TEMPSTR='LOOP:'
            IF (LEN_TRIM(OUTSTR) > 0 ) THEN
              OUTSTR=TRIM(OUTSTR)//' '//TEMPSTR
            ELSE
              OUTSTR=TRIM(OUTSTR)//TEMPSTR
            END IF      
            WRITE(TEMPSTR,'(G15.7)')HARR(I)
            OUTSTR=TRIM(OUTSTR)//ADJUSTL(TEMPSTR)
          END IF  
        CASE(5)
          IF (.NOT. LNULL(HARR(6),HARR(7),HARR(8))) THEN
            IF (ABS(HARR(6))< 0.001) THEN
              IF(ABS(HARR(5)-HARR(7)) < 0.001) THEN
                TEMPSTR='LSIDE:'
                IF (LEN_TRIM(OUTSTR) > 0 ) THEN
                  OUTSTR=TRIM(OUTSTR)//' '//TEMPSTR
                ELSE
                  OUTSTR=TRIM(OUTSTR)//TEMPSTR
                END IF      
                WRITE(TEMPSTR,'(G15.7)')HARR(I)
                OUTSTR=TRIM(OUTSTR)//ADJUSTL(TEMPSTR)
              ELSEIF ( .NOT. LNULL(HARR(6))) THEN
                TEMPSTR='LSIDEX:'
                IF (LEN_TRIM(OUTSTR) > 0 ) THEN
                  OUTSTR=TRIM(OUTSTR)//' '//TEMPSTR
                ELSE
                  OUTSTR=TRIM(OUTSTR)//TEMPSTR
                END IF      
                WRITE(TEMPSTR,'(G15.7)')HARR(I)
                OUTSTR=TRIM(OUTSTR)//ADJUSTL(TEMPSTR)
                TEMPSTR='LSIDEY:'
                IF (LEN_TRIM(OUTSTR) > 0 ) THEN
                  OUTSTR=TRIM(OUTSTR)//' '//TEMPSTR
                ELSE
                  OUTSTR=TRIM(OUTSTR)//TEMPSTR
                END IF      
                WRITE(TEMPSTR,'(G15.7)')HARR(7)
                OUTSTR=TRIM(OUTSTR)//ADJUSTL(TEMPSTR)
              END IF  
            ELSE
              TEMPSTR='LV1X:'
              IF (LEN_TRIM(OUTSTR) > 0 ) THEN
                OUTSTR=TRIM(OUTSTR)//' '//TEMPSTR
              ELSE
                OUTSTR=TRIM(OUTSTR)//TEMPSTR
              END IF      
              WRITE(TEMPSTR,'(G15.7," LV1Y:")')HARR(5)
              OUTSTR=TRIM(OUTSTR)//ADJUSTL(TEMPSTR)
              WRITE(TEMPSTR,'(G15.7," LV2X:")')HARR(7)
              OUTSTR=TRIM(OUTSTR)//ADJUSTL(TEMPSTR)
              WRITE(TEMPSTR,'(G15.7," LV2Y:")')HARR(6)
              OUTSTR=TRIM(OUTSTR)//ADJUSTL(TEMPSTR)
              WRITE(TEMPSTR,'(G15.7," LV3X:")')HARR(7)
              OUTSTR=TRIM(OUTSTR)//ADJUSTL(TEMPSTR)
              WRITE(TEMPSTR,'(G15.7," LV3Y:")')HARR(6)
              OUTSTR=TRIM(OUTSTR)//ADJUSTL(TEMPSTR)
              WRITE(TEMPSTR,'(G15.7," LV4X:")')HARR(8)
              OUTSTR=TRIM(OUTSTR)//ADJUSTL(TEMPSTR)
              WRITE(TEMPSTR,'(G15.7," LV4Y:")')HARR(5)
              OUTSTR=TRIM(OUTSTR)//ADJUSTL(TEMPSTR)
              WRITE(TEMPSTR,'(G15.7)')HARR(8)
              OUTSTR=TRIM(OUTSTR)//ADJUSTL(TEMPSTR)
            END IF
          ELSE
            CYCLE
          END IF
        CASE (6:8)
          CYCLE
        CASE DEFAULT
          WRITE(TEMPSTR,'(A,":")')TRIM(HEADNUM(I))
          IF (LEN_TRIM(OUTSTR) > 0 ) THEN
            OUTSTR=TRIM(OUTSTR)//' '//TEMPSTR
          ELSE
            OUTSTR=TRIM(OUTSTR)//TEMPSTR
          END IF      
          WRITE(TEMPSTR,'(G15.7)')HARR(I)
          OUTSTR=TRIM(OUTSTR)//ADJUSTL(TEMPSTR)
      END SELECT
      IF (LEN_TRIM(OUTSTR) > 80) THEN
        WRITE(LUHOUT,'(A)')TRIM(OUTSTR)
        OUTSTR=''
      END IF
    END IF
  END DO
  IF (TRIM(LABEL(IANG)) == 'HOLE_ANGLES' .AND. .NOT. LNULL(ZMAX(IANG))) THEN
    OUTSTR=TRIM(OUTSTR)//" NORTHTYPE:GRID"
  END IF  
  IF (.NOT. LNULL(HARR(14))) THEN
    WRITE(TEMPSTR,'(F6.1)')1000.0/(HARR(14)*4.0)     !TIME IN MSEC
    OUTSTR=TRIM(OUTSTR)//" ONTIME:"//TRIM(ADJUSTL(TEMPSTR))//" OFFTIME:"//TRIM(ADJUSTL(TEMPSTR))
  END IF 
  !NOTE SUPPORTING FILES
  IF (ISUPFILE > 0) THEN
    IF(LEN_TRIM(BOUNDFILE) > 0) OUTSTR=TRIM(OUTSTR)//" BOUNDFILE:"//TRIM(ADJUSTL(SUPPFILE))
    IF(LEN_TRIM(CHANFILE) > 0) OUTSTR=TRIM(OUTSTR)//" CHANNELFILE:"//TRIM(ADJUSTL(SUPPFILE))
    IF(LEN_TRIM(COLFILE) > 0) OUTSTR=TRIM(OUTSTR)//" COLLARFILE:"//TRIM(ADJUSTL(SUPPFILE))
    IF(LEN_TRIM(SURVFILE) > 0) OUTSTR=TRIM(OUTSTR)//" SURVEYFILE:"//TRIM(ADJUSTL(SUPPFILE))
    IF(LEN_TRIM(WIREFILE) > 0) OUTSTR=TRIM(OUTSTR)//" WIREFILE:"//TRIM(ADJUSTL(SUPPFILE))
    IF(LEN_TRIM(WAVEFFILE) > 0) OUTSTR=TRIM(OUTSTR)//" WAVEFORM:"//TRIM(ADJUSTL(SUPPFILE))
    END IF
  END IF
  WRITE(LUHOUT,'(A)')TRIM(OUTSTR)
  !CREATE RECORD TYPE 3
  IF (NCHAN > 0) THEN
    OUTSTR='@DELAY='
    DO I=1,NCHAN
      SELECT CASE (INT(LOG10(WTIME(I))))
        CASE(:-4)
          WRITE(TEMPSTR,'(F8.7)')WTIME(I)
        CASE(-3)
          WRITE(TEMPSTR,'(F8.6)')WTIME(I)
        CASE(-2)
          WRITE(TEMPSTR,'(F8.5)')WTIME(I)
        CASE(-1)
          WRITE(TEMPSTR,'(F8.4)')WTIME(I)
        CASE(0:)
          WRITE(TEMPSTR,'(F8.3)')WTIME(I)
      END SELECT    
      IF (I > 1) THEN
        OUTSTR=TRIM(OUTSTR)//','//TRIM(ADJUSTL(TEMPSTR))
      ELSE
        OUTSTR=TRIM(OUTSTR)//TRIM(ADJUSTL(TEMPSTR))
      END IF      
    END DO
  END IF
  WRITE(LUHOUT,'(A)')TRIM(OUTSTR)
  IOUTFLD(1:2)=1
  IOUTFLD(ICOMP:ICURR)=1
  IOUTFLD(11:10+NCHAN)=1
  !CREATE RECORD TYPE 4
  OUTSTR='LINE STATION'
  IF (.NOT. LNULL(ZMAX(3),ZMAX(4))) THEN
    IF (IOUTFLD(3) < 1) THEN
      OUTSTR=TRIM(OUTSTR)//' EAST'
      IOUTFLD(3)=1
    ELSE  
      OUTSTR=TRIM(OUTSTR)//' '//ESFLABEL(IOUTFLD(3))
    END IF
    IF (IOUTFLD(4) < 1) THEN
      OUTSTR=TRIM(OUTSTR)//' NORTH'
      IOUTFLD(4)=1
    ELSE  
      OUTSTR=TRIM(OUTSTR)//' '//ESFLABEL(IOUTFLD(4))
    END IF
  ELSE
    IOUTFLD(3)=0
    IOUTFLD(4)=0
  END IF
  IF (.NOT. LNULL(ZMAX(IELEV))) THEN
    IF (IOUTFLD(IELEV) < 1) THEN
      IF (LABEL(IELEV) /= 'ELEVATION   ') THEN
        WRITE (OUTMESS,'("The label ",A," has been found in field ",I2," when ELEVATION &
         &was expected. If what is in this field really is elevation press Yes if not press No and &
         &the existing field label will be used for output")')TRIM(LABEL(IELEV)),IELEV
        CALL WYESNOBOX(OUTMESS,OUTSUB,IACTION)
        IF (IACTION == 1) THEN
          OUTSTR=TRIM(OUTSTR)//' ELEVATION'
        ELSE  
          OUTSTR=TRIM(OUTSTR)//' '//LABEL(IELEV)
        END IF
      ELSE   
        OUTSTR=TRIM(OUTSTR)//' ELEVATION'
      END IF  
      IOUTFLD(5)=1
    ELSE  
      OUTSTR=TRIM(OUTSTR)//' '//ESFLABEL(IOUTFLD(5))
    END IF
  ELSE
    IOUTFLD(5)=0
  END IF
  IF (TRIM(LABEL(IANG)) == 'HOLE_ANGLES' .AND. .NOT. LNULL(ZMAX(IANG))) THEN
    OUTSTR=TRIM(OUTSTR)//' AZIMUTH DIP'
    IOUTFLD(IANG)=1
  ELSEIF (.NOT. LNULL(ZMAX(IANG))) THEN
    IF (IOUTFLD(IANG) < 1) THEN
      OUTSTR=TRIM(OUTSTR)//' '//LABEL(IANG)
      IOUTFLD(IANG)=1
    ELSE  
      OUTSTR=TRIM(OUTSTR)//' '//ESFLABEL(IOUTFLD(IANG))
    END IF
  ELSE
    IOUTFLD(IANG)=0
  END IF    
  OUTSTR=TRIM(OUTSTR)//' COMPONENT CURRENT'
  IF (.NOT. LNULL(ZMAX(ICURR+1))) THEN
    IF (IOUTFLD(ICURR+1) < 1) THEN
      OUTSTR=TRIM(OUTSTR)//' '//LABEL(ICURR+1)
      IOUTFLD(ICURR+1)=1
    ELSE
      OUTSTR=TRIM(OUTSTR)//' '//ESFLABEL(IOUTFLD(ICURR+1))
    END IF  
  ELSE
    IOUTFLD(ICURR+1)=0
  END IF    
  IF (.NOT. LNULL(ZMAX(ICURR+2))) THEN
    IF (IOUTFLD(ICURR+2) < 1) THEN
      OUTSTR=TRIM(OUTSTR)//' '//LABEL(ICURR+2)
      IOUTFLD(ICURR+2)=1
    ELSE
      OUTSTR=TRIM(OUTSTR)//' '//ESFLABEL(IOUTFLD(ICURR+2))
    END IF  
  ELSE
    IOUTFLD(ICURR+2)=0
  END IF    
  IF (NCHAN > 0) THEN
    DO I=1,NCHAN
      OUTSTR=TRIM(OUTSTR)//' '//LABEL(10+I)(1:4)
    END DO
!  ELSE
!    DO I=1,NHARM
!      WRITE(TEMPSTR,'("MAG",I1," PH",I1)')I,I
!      OUTSTR=TRIM(OUTSTR)//' '//TRIM(TEMPSTR)
!    END DO
  END IF
  DO I=11+NCHAN,NFLD
    IF (IOUTFLD(I) > 0) THEN
      IF (IOUTFLD(I) > 1) THEN 
        OUTSTR=TRIM(OUTSTR)//' '//ESFLABEL(IOUTFLD(I))
      ELSE
        OUTSTR=TRIM(OUTSTR)//' '//LABEL(I)
      END IF  
    END IF
  END DO
  WRITE(LUHOUT,'(A)')TRIM(OUTSTR)
  RETURN
  END SUBROUTINE ESFEMHEAD
  !%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  SUBROUTINE CREATESUPPORT()
  INTEGER :: LUHINSUPF
  INTEGER :: NPOINTS
  INTEGER :: I
  INTEGER :: IERR
  REAL :: HOLE,X,Y,Z
  CHARACTER(LEN=80) :: ALINE
  CHARACTER(LEN=255) :: OUTERR
  CHARACTER(LEN=20), PARAMETER :: OUTSUB='CREATESUPPORT'


  IF(LEN_TRIM(BOUNDFILE) > 0) THEN
    CALL GETHANDL(LUHINSUPF)
    OPEN(UNIT=LUHINSUPF,FILE=BOUNDFILE,ACCESS='SEQUENTIAL',FORM='FORMATTED',STATUS='OLD')
    READ(LUHINSUPF,*,IOSTAT=IERR,IOMSG=OUTERR) NPOINTS
    IF(IERR /= 0) THEN
      WRITE(OUTMESS,'("PROBLEM READING BOUNDARY BLN FILE. ERROR= ",A)')TRIM(OUTERR)
      CALL WERRORBOX(OUTMESS,OUTSUB)
    ELSE
      IF (ISUPFILE > 1) WRITE(LUHSUPF,'("BOUNDFILE")')
      WRITE(LUHSUPF,'("BOUNDARY FILE CREATED FROM ",A)')TRIM(BOUNDFILE)
      WRITE(LUHSUPF,'("NUMBOUND=",I5)')NPOINTS
      WRITE(LUHSUPF,'("LONG   LAT")')
      DO I=1,NPOINTS
        READ(LUHINSUPF,'(A)',IOSTAT=IERR,IOMSG=OUTERR)ALINE
        SELECT CASE (IERR)
        CASE (:-1)
          OUTMESS='UNEXPECTED END OF BOUNDARY FILE - CHECK THAT NUMBER OF VERTICES SPECIFIED CORRECTLY'
          CALL WERRORBOX(OUTMESS,OUTSUB)
        CASE (1:)
          WRITE(OUTMESS,'("PROBLEM READING BOUNDARY BLN FILE. ERROR= ",A)')TRIM(OUTERR)
          CALL WERRORBOX(OUTMESS,OUTSUB)
        CASE (0)
          WRITE(LUHSUPF,'(A)')ALINE
        END SELECT
      END DO
    END IF
    CLOSE(LUHINSUPF)
  END IF
  IF(LEN_TRIM(CHANFILE) > 0) THEN
    IF (ISUPFILE > 1) THEN
      WRITE(LUHSUPF,'("CHANNELFILE")')
      CALL GETHANDL(LUHINSUPF)
      OPEN(UNIT=LUHINSUPF,FILE=BOUNDFILE,ACCESS='SEQUENTIAL',FORM='FORMATTED',STATUS='OLD')
      DO
        READ(LUHINSUPF,'(A)',IOSTAT=IERR,IOMSG=OUTERR)ALINE
        SELECT CASE (IERR)
        CASE (:-1)
          EXIT
        CASE (1:)
          WRITE(OUTMESS,'("PROBLEM READING CHANNEL FILE. ERROR= ",A)')TRIM(OUTERR)
          CALL WERRORBOX(OUTMESS,OUTSUB)
        CASE (0)
          WRITE(LUHSUPF,'(A)')ALINE
        END SELECT
      END DO
      CLOSE(LUHINSUPF)
    END IF
  END IF
  IF(LEN_TRIM(COLFILE) > 0) THEN
    IF (ISUPFILE > 1) WRITE(LUHSUPF,'("COLLARFILE")')
    WRITE(LUHSUPF,'("COLLAR FILE FOR ",A)')TRIM(FILEOUT)
    CALL GETHANDL(LUHINSUPF)
    OPEN(UNIT=LUHINSUPF,FILE=COLFILE,ACCESS='SEQUENTIAL',FORM='FORMATTED',STATUS='OLD')
    ! READ COLLAR FILE THROUGH ONCE TO DETERMINE MAX NUMBER OF HOLES
    !
    NPOINTS=0
    DO
      READ(LUHINSUPF,'(A)',IOSTAT=IERR,IOMSG=OUTERR) ALINE
      SELECT CASE (IERR)
        CASE (:-1)
          EXIT
        CASE (1:)
          OUTMESS='PROBLEM READING COLLAR FILE'
          CALL WERRORBOX(OUTMESS,OUTSUB)
        CASE (0)  ! NO ERROR - PROCESS LINE
          IF (LEN_TRIM(ALINE) > 0) NPOINTS = NPOINTS + 1
      END SELECT
    END DO
    REWIND(LUHINSUPF)
    WRITE(LUHSUPF,'("NUMHOLES=",I5)')NPOINTS
    WRITE(LUHSUPF,'("HOLE EAST NORTH ELEVATION")')
    DO I=1,NPOINTS
      READ (LUHINSUPF,*,IOSTAT=IERR,IOMSG=OUTERR) X,Y,Z
      IF (IERR /=0) THEN
        WRITE(OUTMESS,'("PROBLEM READING COLLAR FILE. ERROR= ",A)')TRIM(OUTERR)
        CALL WERRORBOX(OUTMESS,OUTSUB)
      ELSE
        WRITE(LUHSUPF,'(3G15.7)') X,Y,Z
      END IF
    END DO
    CLOSE(LUHINSUPF)
  END IF
  IF(LEN_TRIM(SURVFILE) > 0) THEN
    IF (ISUPFILE > 1) WRITE(LUHSUPF,'("SURVEYFILE")')
    WRITE(LUHSUPF,'("SURVEY FILE FOR ",A)')TRIM(FILEOUT)
    CALL GETHANDL(LUHINSUPF)
    OPEN(UNIT=LUHINSUPF,FILE=SURVFILE,ACCESS='SEQUENTIAL',FORM='FORMATTED',STATUS='OLD')
    ! READ SURVEY FILE THROUGH ONCE TO DETERMINE NUMBER OF POINTS
    !
    NPOINTS=0
    DO
      READ(LUHINSUPF,'(A)',IOSTAT=IERR,IOMSG=OUTERR) ALINE
      SELECT CASE (IERR)
        CASE (:-1)
          EXIT
        CASE (1:)
          OUTMESS='PROBLEM READING SURVEY FILE'
          CALL WERRORBOX(OUTMESS,OUTSUB)
        CASE (0)  ! NO ERROR - PROCESS LINE
          IF (LEN_TRIM(ALINE) > 0) NPOINTS = NPOINTS + 1
      END SELECT
    END DO
    REWIND(LUHINSUPF)
    WRITE(LUHSUPF,'("NSURV=",I5)')NPOINTS
    WRITE(LUHSUPF,'("HOLE DEPTH AZIMUTH DIP")')
    DO I=1,NPOINTS
      READ (LUHINSUPF,*,IOSTAT=IERR,IOMSG=OUTERR)HOLE,X,Y,Z
      IF (IERR /=0) THEN
        WRITE(OUTMESS,'("PROBLEM READING SURVEY FILE. ERROR= ",A)')TRIM(OUTERR)
        CALL WERRORBOX(OUTMESS,OUTSUB)
      ELSE
        Z=-Z
        WRITE(LUHSUPF,'(4G15.7)')HOLE,X,Y,Z
      END IF
    END DO
    CLOSE(LUHINSUPF)
  END IF
  IF(LEN_TRIM(WIREFILE) > 0) THEN
    CALL GETHANDL(LUHINSUPF)
    OPEN(UNIT=LUHINSUPF,FILE=WIREFILE,ACCESS='SEQUENTIAL',FORM='FORMATTED',STATUS='OLD')
    READ(LUHINSUPF,*,IOSTAT=IERR,IOMSG=OUTERR) NPOINTS
    IF(IERR /= 0) THEN
      WRITE(OUTMESS,'("PROBLEM READING TX WIRE BLN FILE. ERROR= ",A)')TRIM(OUTERR)
      CALL WERRORBOX(OUTMESS,OUTSUB)
    ELSE
      IF (ISUPFILE > 1) WRITE(LUHSUPF,'("WIREFILE")')
      WRITE(LUHSUPF,'("WIRE FILE CREATED FROM ",A)')TRIM(WIREFILE)
      WRITE(LUHSUPF,'("NVERT=",I5)')NPOINTS
      WRITE(LUHSUPF,'("EAST NORTH ELEVATION")')
      DO I=1,NPOINTS
        READ(LUHINSUPF,'(A)',IOSTAT=IERR,IOMSG=OUTERR)ALINE
        SELECT CASE (IERR)
        CASE (:-1)
          OUTMESS='UNEXPECTED END OF BOUNDARY FILE - CHECK THAT NUMBER OF VERTICES SPECIFIED CORRECTLY'
          CALL WERRORBOX(OUTMESS,OUTSUB)
        CASE (1:)
          WRITE(OUTMESS,'("PROBLEM READING TX WIRE BLN FILE. ERROR= ",A)')TRIM(OUTERR)
          CALL WERRORBOX(OUTMESS,OUTSUB)
        CASE (0)
          WRITE(LUHSUPF,'(A)')ALINE
        END SELECT
      END DO
    END IF
    CLOSE(LUHINSUPF)
  END IF
  IF(LEN_TRIM(WAVEFFILE) > 0) THEN
    IF (ISUPFILE > 1) THEN
      WRITE(LUHSUPF,'("WAVEFORM")')
      CALL GETHANDL(LUHINSUPF)
      OPEN(UNIT=LUHINSUPF,FILE=WAVEFFILE,ACCESS='SEQUENTIAL',FORM='FORMATTED',STATUS='OLD')
      DO
        READ(LUHINSUPF,'(A)',IOSTAT=IERR,IOMSG=OUTERR)ALINE
        SELECT CASE (IERR)
        CASE (:-1)
          EXIT
        CASE (1:)
          WRITE(OUTMESS,'("PROBLEM READING WAVEFORM FILE. ERROR= ",A)')TRIM(OUTERR)
          CALL WERRORBOX(OUTMESS,OUTSUB)
        CASE (0)
          WRITE(LUHSUPF,'(A)')ALINE
        END SELECT
      END DO
      CLOSE(LUHINSUPF)
    END IF
  END IF
  CLOSE(LUHSUPF)
  RETURN
  END SUBROUTINE CREATESUPPORT
END SUBROUTINE EMTESF
